$title sr model (srmodel.gms)
* This version of the model developed at the U.S. International
* Trade Commission for analysis of U.S. trade policy.  It has a
* variable labor supply.  The model closure involves fixed real investment
* demands, fixed real government demands, a fixed price index, and variable
* government transfer.  The exchange rate is the real exchange rate.
$offsymlist offsymxref

options decimals=3;

* ## read in social accounting matrix and behavioral parameter dataset

$include sam.inc
$include par.inc


* ## define sets

sets
      ie(i)       export sectors
      ied(i)      sectors with export demand eqn
      im(i)       import sectors
      imnq(i)     import sectors without quotas
      ims(i)      sectors with an import supply function
      ine(i)      non-exportable sectors
      ined(i)     sectors without an export demand function
      inm(i)      non-importable sectors
      inms(i)     sectors without an import supply function
      inq(i)      sectors without import quotas
      int(i)      non traded sectors
      iq(i)       sectors with import quotas
      iqd(i)      sectors with domestic held import quotas
      iqf(i)      sectors with foreign held import quotas
      it(i)       traded sectors
      itr(i)      wholesale and retail trade sector / trade /
      nwim(i)     armington import sectors
      nwie(i)     armington export sectors
      ;


* ## declare model parameters ##

* structural parameters

parameters

   aa             exponential product of marginal budget shares
   aggsub         aggregate subsistence minimum for nested les
   alpha(i)       armington function shift parameter
   ax(i)          intercept parameter in production function
   beta(i)        armington function share parameter
   bx(i)          share parameter in production function
   check          checking parameter for marginal budget shares in les
   delta(i)       cet function share parameter
   econst(i)      export demand constant
   engel          adjustment parameter for engel aggregation condition
   gamma(i)       cet function shift parameter
   gs(i)          government consumption shares
   io(i,j)        input-output coefficiencts
   is(i)          private investment shares
   itax(i)        indirect tax rates
   ktax           capital tax rate
   ltax           labor tax rate
   maxlab         maximum labor supply
   mbsh(i)        marginal budget shares for goods
   mbshl          marginal budget share for leisure
   mconst(i)      import supply constant
   mps            marginal propensity to save
   pelas(i)       LES own price elasticities
   phi(i)         ces parameter in production function
   qd(i)          domestic quota switch parameter
   rdist(i)       rental proportionality factors
   sig(i)         armington function elasticity
   sigf(i)        import supply elasticity
   sub(i)         subsistence minima for goods
   subl           subsistence minimum for leisure
   tau(i)         cet function elasticity
   tauf(i)        export demand price elasticity
   te(i)          export subsidy rates
   thetad(i)      share of domestic quota premia accruing to foreigners
   thetaf(i)      share of foreign quota premia accruing to foreigners
   tm(i)          tariff rates on imports
   wdist(i)       sectoral wage distribution
   yelas(i)       income elasticity of consumption
   ytax           income tax rate


* parameters to initialize variables

   cd0(i)         household final demand
   ds0(i)         domestic sales
   e0(i)          exports by sector
   dd0(i)         domestic demand
   fsav0          net foreign savings
   gd0(i)         final demand by government
   gdtot0         total government demand
   gfbor          total government foreign borrowing
   govsav0        government saving
   gtran0         government transfers
   gdpr0          real gdp
   gdpva0         value added in market prices gdp
   id0(i)         final investment demand
   inter0(i)      intermediate demand
   invest0        total investment
   itaxt0         indirect tax revenue
   kd0(i)         capital stocks by sector
   ks0            aggregate capital stock
   ktaxt0         total capital taxes
   ld0(i)         employment by sector
   lesp0          les price index for goods
   ls0            labor supply
   ltaxt0         total labor taxes
   m0(i)          import demand
   netsub0        export subsidies
   omega0         objective function variable
   psave0         total private saving
   px0(i)         price of composite goods
   pd0(i)         domestic prices
   pe0(i)         domestic price of exports
   pindex0        gdp deflator
   pm0(i)         domestic price of imports
   pq0(i)         sales price
   pva0(i)        value added price by sector
   pwe0(i)        world market price of exports
   pwm0(i)        world market price of imports
   q0(i)          domestic demand by sector
   qrentd0(i)     quota rents for domestic held quota
   qrentf0(i)     quota rents for foreign held quota
   ra0            economywide average rental rate
   remit          foreign remittances
   rhod0(i)       tariff equivalent domestic held quota rate
   rhof0(i)       tariff equivalent foreign held quota rate
   savings0       total savings
   tariff0        tariff revenue
   x0(i)          volume of domestic output by sector
   wa0            average wage rate
   y0             private gnp
   ycap0          gross capital income
   ylab0          gross labor income
   ytaxt0         total income tax
   ;


* ## declare variables and equations ##

variables

   cd(i)         household demand (cdeq)
   ds(i)         domestic sales (cet dsnt)
   e(i)          exports by sector (esupply e fx)
   er            real exchange rate (bop)
   fsav          foreign savings (fsav fx)
   gd(i)         government final demand by sector (gdeq)
   gdtot         total government demand (govt fx)
   gdpr          real gdp (gdpreal)
   gdpva         value added in market prices gdp (gdpvad)
   govsav        government savings (govt)
   gtran         government transfers (pindeq)
   id(i)         private investment demand by sector (ideq)
   itaxt         indirect tax revenue (itaxeq)
   inter(i)      intermediates uses (intermed)
   invest        total investment (invest fx)
   kd(i)         capital stock by sector (kdeq)
   ks            capital supply (ks fx)
   ktaxt         total capital taxes (ktaxeq)
   ld(i)         employment by sector (ldeq)
   lesp          linear expenditure system price (lespeq)
   ls            labor supply (lsup)
   ltaxt         total labor taxes (ltaxeq)
   m(i)          import demand (armington m fx)
   netsub        export subsidies (subeq)
   omega         objective function variable (obj)
   pd(i)         domestic prices (costmin qnt)
   pe(i)         domestic price of exports (pedef pe fx)
   pindex        gdp deflator (pindex fx)
   pm(i)         domestic price of imports (pmdefnq pmdefqd pmdefqf pm fx)
   pq(i)         average output price by sector (absorption)
   pva(i)        value added price by sector (actp)
   pwe(i)        world price of exports (edemand pwe fx)
   pwm(i)        world market price of imports (msupply pwm fx)
   px(i)         price of composite goods (sales)
   q(i)          domestic demand by sector (gmarket)
   qrentd(i)     quota rents for domestic held quota (qrentdeq)
   qrentf(i)     quota rents for foreign held quota (qrentfeq)
   ra            economywide average rental rate (kmarket)
   rhod(i)       tariff equivalent for domestic quota (rhod fx)
   rhof(i)       tariff equivalent for foreign quota (rhof fx)
   savings       total savings (saveq)
   tariff        tariff revenue (tareq)
   wa            average wage rate by labor category (lmarket)
   walras        savings-investment balance (walraseq)
   x(i)          composite goods supply (product)
   y             private gnp (gnp)
   ycap          gross capital income (capitaly)
   ylab          gross labor income (labory)
   ytaxt         income tax revenue (ytaxeq)
   ;


equations

   absorption(i) value of domestic sales
   actp(i)       definition of activity prices
   arma(i)       composite good aggregation for armington sectors    
   bop           balance of payments
   capitaly      gross capital income
   cdeq(i)       private consumption behavior
   cet(i)        cet function
   costmina(i)   f.o.c. for cost minimization of armington composite good   
   dsnt(i)       domestic supply for nontradeable sectors
   edemand(i)    export demand functions
   esupply(i)    export supply
   gdeq(i)       government final demand by sector
   gnp           private gnp
   gmarket(i)    goods markets equilibrium
   gdpvad        total value added including indtax
   gdpreal       real gdp
   govt          government budget constraint
   ideq(i)       private investment demand by sector
   intermed(i)   sectoral intermediate demand
   itaxeq        indirect taxes on domestic production
   kdeq(i)       capital demand
   kmarket       capital market equilibrium
   ktaxeq        capital taxation
   labory        total income accruing to labor
   ldeq(i)       labor demand
   lespeq        aggregate les price index
   lsup          labor supply
   ltaxeq        labor taxation
   lmarket       labor market equilibrium
   msupply(i)    upward sloping supply curve for imports
   obj           objective function
   pedef(i)      definition of domestic export prices
   pindeq        price index equation
   pmdefnq(i)    definition of domestic import prices with no quota
   pmdefqd(i)    definition of domestic import prices with domestic quota
   pmdefqf(i)    definition of domestic import prices with foreign quota
   product(i)    cet domestic production function
   qnt(i)        composite demand for nontradeable sectors
   qrentdeq(i)   domestic held quota rents equation
   qrentfeq(i)   foreign held quota rents equation
   sales(i)      value of domestic output
   saveq         total savings
   subeq         export subsidies
   tareq         tariff revenue
   walraseq      walras law equation
   ytaxeq        income tax
   ;


* ## assign initial variable values ##

* demands, imports, and outputs

 inter0(i) = sum(j,sam(i,j));
 cd0(i)    = sam(i,'household');
 gd0(i)    = sam(i,'government');
 id0(i)    = sam(i,'capaccount');
 e0(i)     = sam(i,'row');
 m0(i)     = sam('row',i) + sam('rowtaxes',i);
 rhod0(i)  = param(i,'rhod');
 rhof0(i)  = param(i,'rhof');
 tm(i) $ m0(i)         = sam('rowtaxes',i)/(m0(i)-sam('rowtaxes',i));
 qrentf0(i) $ rhof0(i) = (rhof0(i)/((1+tm(i))*(1+rhof0(i))))*m0(i);
 qrentd0(i) $ rhod0(i) = ((rhod0(i)/(1+rhod0(i)))*m0(i)) - sam('rowtaxes',i);
 qrentd0(i) $ (rhod0(i) and (qrentd0(i) le 0)) = 0;
 rhod0(i) $ (qrentd0(i) eq 0) = 0;
 qd(i) $ rhod0(i) = 1;
 tm(i) $ rhod0(i) = sam('rowtaxes',i)/(sam('row',i) - qrentd0(i));
 thetad(i) = param(i,'thetad');
 thetaf(i) = param(i,'thetaf');
 dd0(i)    = sum(j,sam(i,j)) + sam(i,'household')
                 + sam(i,'government') + sam(i,'capaccount') ;
 netsub0   = 0;
 tariff0   = sum(i,sam('rowtaxes',i));
 x0(i)     = dd0(i) + e0(i) - m0(i);
 ds0(i)    = x0(i) - e0(i);
 q0(i)     = ds0(i) + m0(i);

 display m0, tm, rhod0, rhof0, qrentd0, qrentf0, qd;

* factors

 kd0(i)  = param(i,'capital');
 ks0     = sum(i,kd0(i));
 ra0     = sum(i,sam('property',i))/ks0;
 ld0(i)  = param(i,'employed');
 ls0     = sum(i,ld0(i));
 wa0     = sum(i,sam('labor',i))/ls0;


*## initialize sectoral indices

 ie(i)   = yes $ e0(i);
 ied(i)  = yes $ (param(i,'tauf') gt -20 and param(i,'tauf') lt 0);
 ined(i) = not ied(i);
 ims(i)  = yes $ (param(i,'sigf') lt 20 and param(i,'sigf') gt 0);
 inms(i) = not ims(i);
 im(i)   = yes $ m0(i);
 inm(i)  = not im(i);
 ine(i)  = not ie(i);
 iq(i)   = yes $ ((rhod0(i) gt 0) or (rhof0(i) gt 0));
 iqd(i)  = yes $ (rhod0(i) gt 0);
 iqf(i)  = yes $ (rhof0(i) gt 0);
 inq(i)  = not iq(i);
 imnq(i) = im(i)*inq(i);
 it(i)   = ie(i) + im(i);
 int(i)  = not it(i);

* setting up Armington import and export sector indexes

 nwim(i) = im(i);
 nwie(i) = ie(i);


* ## initialization of exogenous parameters ##

 yelas(i) = param(i,'yelas');
 phi(i)   = param(i,'phi');
 sig(i)   = param(i,'sig');
 sigf(i)  = param(i,'sigf')$(param(i,'sigf') lt 20);
 tau(i)   = param(i,'tau');
 tauf(i)  = param(i,'tauf')$(param(i,'tauf') gt -20);
 wdist(i) = sam('labor',i)/(wa0*ld0(i));
 rdist(i) = sam('property',i)/(ra0*kd0(i));
 itaxt0   = sum(i,sam('government',i));
 io(i,j)  = (sam(i,j))/x0(j);
 te(ie)   = 0;
 gdtot0   = sum(i,sam(i,'government'));
 gs(i)    = sam(i,'government')/gdtot0;
 invest0  = sum(i,sam(i,'capaccount'));
 is(i)    = sam(i,'capaccount')/invest0;


* ## initialize prices ##

 pd0(i)   = 1;
 pm0(it)  = 1;
 pm0(int) = 0;
 pe0(ie)  = 1;
 pe0(ine) = 0;
 pwm0(it) $ rhod0(it) = pm0(it)/((1+rhod0(it))*er0);
 pwm0(it) $ rhof0(it) = pm0(it)/((1+tm(it))*(1+rhof0(it))*er0);
 pwm0(it) $ inq(it) = pm0(it)/((1+tm(it))*er0);
 pwm0(int) = 0;
 pwe0(ie)  = pe0(ie)/((1 + te(ie))*er0);
 pwe0(ine) = 0;
 pq0(i)    = (pd0(i)*ds0(i) + (pm0(i)*m0(i))$im(i))/q0(i);
 px0(i)    = (pd0(i)*ds0(i) + (pe0(i)*e0(i))$ie(i))/x0(i);
 itax(i)   = sam('government',i)/(px0(i)*x0(i));
 pva0(i)   = px0(i)*(1.-itax(i)) - sum(j,io(j,i)*pq0(j));


* ## initialize macro variables ##

 gdpr0 = sum(i, cd0(i)+is(i)*invest0+gs(i)*gdtot0+e0(i)-er0*pwm0(i)*m0(i) )
                    - sum(iqd, thetad(iqd)*qrentd0(iqd) )
                    - sum(iqf, thetaf(iqf)*qrentf0(iqf) ) ;
 gdpva0 = sum(i,pva0(i)*x0(i)) + itaxt0 + tariff0;
 pindex0 = gdpva0/gdpr0;
 fsav0 = sam('capaccount','row') - sam('row','capaccount');
 gfbor = sam('government','row') - sam('row','government');
 govsav0 = sam('capaccount','government') -
                                        sam('government','capaccount');
 gtran0 = sam('enterprise','government') + sam('household','government');
 ktaxt0 = sam('government','enterprise');
 ltaxt0 = sam('government','labor');
 psave0 = sam('capaccount','enterprise') + sam('capaccount','household');
 remit = sam('enterprise','row') - sam('row','enterprise')
         + sam('household','row') - sam('row','household')
         + sam('property','row') - sam('row','property')
         - sum(j $ rhod0(j), (1-thetad(j))*qrentd0(j))
         - sum(j $ rhof0(j), (1-thetaf(j))*qrentf0(j));
 ytaxt0 =  sam('government','household');
 ycap0 = ra0*ks0;
 ylab0 = wa0*ls0;
 omega0  = ylab0 + ycap0;
 ltax = ltaxt0/ylab0;
 ktax = ktaxt0/ycap0;
 y0 = ylab0 + ycap0 + gtran0 + er0*remit
          + sum(j $ rhod0(j),(1-thetad(j))*qrentd0(j))
          + sum(j $ rhof0(j),(1-thetaf(j))*qrentf0(j));
 ytax = ytaxt0/(y0-ltaxt0-ktaxt0);
 mps = psave0/(y0-ytaxt0-ltaxt0-ktaxt0);
 savings0 = psave0 + govsav0 + fsav0;


* ## calibrate les for personal consumption and labor supply ##

* rescale income elasticities for the engel aggregation condition

  engel = sum(i, yelas(i)*pq0(i)*cd0(i))/((1-mps)*(1-ytax)
                                                  *(y0-ltaxt0-ktaxt0));
  yelas(i) = yelas(i)/engel;

* calculate and rescale marginal budget shares

  mbsh(i) = yelas(i)*pq0(i)*cd0(i)/((1-mps)*(1-ytax)*(y0-ltaxt0-ktaxt0));
  check = sum(i,mbsh(i));
  mbsh(i) = mbsh(i)/check;

* calculate exponential product parameter

  aa = prod(i$mbsh(i),abs(mbsh(i))**abs(mbsh(i)));

* calculate subsistence minima for goods

  sub(i) = cd0(i) + (mbsh(i)*(1-mps)*(1-ytax)*(y0-ltaxt0-ktaxt0))
           /(frisch*pq0(i));

* calculate marginal budget share for leisure

  mbshl = -wa0*ls0*yelasl/
          (((1-mps)*(1-ytax)*(y0-ltaxt0-ktaxt0))-wa0*ls0*yelasl);

* calculate les price index for goods

  lesp0 = ((1-mps)*(1-ytax)*(y0-ltaxt0-ktaxt0) - sum(i,pq0(i)*sub(i)))/
          (prod(i,(cd0(i)-sub(i))**mbsh(i)));

* calculate aggregate subsistence minimum for nested les

  aggsub = (sum(i,pq0(i)*sub(i)))*(prod(i,((cd0(i)-sub(i))**mbsh(i))))/
           ((1-mps)*(1-ytax)*(y0-ltaxt0-ktaxt0) - sum(i,pq0(i)*sub(i)));

* calculate maximum labor supply

  maxlab = ls0 + (mbshl/wa0)*((1-mps)*(1-ytax)*(y0-ltaxt0-ktaxt0) -
                 lesp0*aggsub)/(1-mbshl);

* calculate price elasticities

  pelas(i) = -yelas(i)*((pq0(i)*sub(i))/((1-mps)*(1-ytax)*(y0-ltaxt0-ktaxt0))
                  - (1/frisch));


display yelas;
display mbsh, sub;
display ls0, mbshl, lesp0, aggsub, maxlab;
display pelas;
                 

* ## calibrate structural parameters ##

* armington import-domestic

 beta(nwim)  = (pm0(nwim)/pd0(nwim))*((m0(nwim)/ds0(nwim))**(1/sig(nwim)));
 beta(nwim)  = beta(nwim)/(1+beta(nwim));      
 alpha(nwim) = q0(nwim)/(beta(nwim)*m0(nwim)**((sig(nwim)-1)/sig(nwim))
                +(1-beta(nwim))*ds0(nwim)**((sig(nwim)-1)/sig(nwim)))      
                  **(sig(nwim)/(sig(nwim)-1));      

* beta(im)  = (pm0(im)/pd0(im))*((m0(im)/ds0(im))**(1/sig(im)));
* beta(im)  = beta(im)/(1+beta(im));
* alpha(im) = q0(im)/(beta(im)*m0(im)**((sig(im)-1)/sig(im))
*                +(1-beta(im))*ds0(im)**((sig(im)-1)/sig(im)))
*                  **(sig(im)/(sig(im)-1));

* cet export-domestic

 delta(ie)  = (pe0(ie)/pd0(ie))*((ds0(ie)/e0(ie))**(1/tau(ie)));
 delta(ie)  = delta(ie)/(1+delta(ie));
 gamma(ie)  = x0(ie)/((delta(ie)*e0(ie)**((tau(ie)+1)/tau(ie))
       +(1-delta(ie))*ds0(ie)**((tau(ie)+1)/tau(ie)))**(tau(ie)/(tau(ie)
                                                                  +1)));
 gamma(ine) = 1.0;

* ces production function

 bx(i) = (wa0*wdist(i)/(ra0*rdist(i)))*(ld0(i)/kd0(i))**(1/phi(i));
 bx(i) = bx(i)/(1+bx(i));
 ax(i) = x0(i)/(bx(i)*ld0(i)**((phi(i)-1)/phi(i))
       + (1-bx(i))*kd0(i)**((phi(i)-1)/phi(i)))**(phi(i)/(phi(i)-1));

* intercepts for export demand and import supply

 econst(i)$ied(i)  = e0(i)/(pwe0(i)**tauf(i)) ;
 mconst(i)$ims(i)  = m0(i)/(pwm0(i)**sigf(i)) ;


* ## display parameters

*display i,im,ie,ied,ims;
*display x0,ds0,q0,inter0,cd0,id0,gd0,m0,e0;
*display ld0,kd0,wdist,rdist;
*display px0,pd0,pq0,pm0,pe0,pva0;
*display gdpr0,gdpva0,ycap0,ylab0,y0,ytaxt0,savings0;
display io,alpha,beta,gamma,delta,ax,bx,phi,
        sig,tau,sigf,tauf,mbsh,econst,mconst;


* ## define equations ##

* final demand behavior

* (household consumption)

   cdeq(i)..       pq(i)*cd(i) =e= pq(i)*sub(i)
                                   + mbsh(i)*((1-mps)*(1-ytax)
                                                        *(y-ltaxt-ktaxt)
                                   - sum(j,pq(j)*sub(j)));

* (investment demand)

   ideq(i)..       id(i)  =e=  is(i)*invest ;

* (government demand)

   gdeq(i)..       gd(i)  =e=  gs(i)*gdtot ;


* production technology

   product(i)..    x(i) =e= ax(i)*(bx(i)*ld(i)**((phi(i)-1)/phi(i))
                             +(1-bx(i))*kd(i)**((phi(i)-1)/phi(i)))
                             **(phi(i)/(phi(i)-1)) ;

   intermed(i)..   inter(i) =e= sum(j,io(i,j)*x(j)) ;


* factor supplies

  lsup..        ls =e= maxlab - (mbshl/wa)*((1-mps)*(1-ytax)*(y-ltaxt-ktaxt)
                              - lesp*aggsub)/(1-mbshl);


  lespeq..      lesp =e= ((1-mps)*(1-ytax)*(y-ltaxt-ktaxt)
                         - sum(i,pq(i)*sub(i)))/
                         (prod(i,(cd(i)-sub(i))**mbsh(i)));


* factor demands

   ldeq(i)..       ld(i) =e= ax(i)**(phi(i)-1)*x(i)
                           *(bx(i)*pva(i)/(wa*wdist(i)))**phi(i) ;

   kdeq(i)..       kd(i) =e= ax(i)**(phi(i)-1)*x(i)
                          *((1-bx(i))*pva(i)/(ra*rdist(i)))**phi(i) ;

*  rental..        ra*ks =e= sum(i,pva(i)*x(i)) - wa*ls ;


* domestic demand and allocation of traded goods


*               Armington Sectors: demand
   arma(nwim)..  q(nwim) =e=   
                    + alpha(nwim)*(beta(nwim)*m(nwim)   
                     **((sig(nwim)-1)/sig(nwim))   
               + (1-beta(nwim))*ds(nwim)**((sig(nwim)-1)/sig(nwim)))   
               **(sig(nwim)/(sig(nwim)-1));   
                                          
*               Armington Sectors: pricing    
   costmina(nwim)..   m(nwim)/ds(nwim) =e=   
           ((pd(nwim)/pm(nwim))*(beta(nwim)/(1-beta(nwim))))**sig(nwim) ;    


   qnt(inm)..      q(inm) =e= ds(inm) ;

   cet(ie)..       x(ie) =e=
                   gamma(ie)*( delta(ie)*e(ie)**((tau(ie)+1)/tau(ie))
                   +(1-delta(ie))*ds(ie)**((tau(ie)+1)/tau(ie)))
                   **(tau(ie)/(tau(ie)+1));

   esupply(ie)..   e(ie)/ds(ie) =e=
                   ((pe(ie)/pd(ie))*((1-delta(ie))/delta(ie)))**(tau(ie));

   dsnt(ine)..     x(ine) =e= ds(ine);


* domestic prices

   actp(i)..       px(i)*(1-itax(i)) =e= pva(i) + sum(j,io(j,i)*pq(j));

   sales(i)..      px(i)*x(i) =e= pd(i)*ds(i) + (pe(i)*e(i))$ie(i);

   absorption(i).. pq(i)*q(i) =e= pd(i)*ds(i) + (pm(i)*m(i))$im(i);


* domestic market equilibrium

   gmarket(i)..    q(i) =e= inter(i) + cd(i) + gd(i) + id(i) ;

   lmarket..       sum(i,ld(i)) =e= ls ;

   kmarket..       sum(i,kd(i)) =e= ks ;


* income and government revenue

   gnp..           y =e= ylab + ycap + gtran + er*remit
                         + sum(iqd, (1-thetad(iqd))*qrentd(iqd) )
                         + sum(iqf, (1-thetaf(iqf))*qrentf(iqf) );

   gdpvad..        gdpva =e= sum(i,pva(i)*x(i)) + itaxt + tariff ;

   gdpreal..       gdpr =e= sum(i, cd(i) + id(i) + gd(i) )
                    + sum(ie, e(ie) ) - sum(im, er*pwm(im)*m(im) )
                    - sum(iqd, thetad(iqd)*qrentd(iqd) )
                    - sum(iqf, thetaf(iqf)*qrentf(iqf) ) ;
   
   labory..        ylab =e= sum(i,wa*wdist(i)*ld(i)) ;

   capitaly..      ycap =e= sum(i,ra*rdist(i)*kd(i));

   govt..          sum(i,pq(i)*gd(i)) + govsav + gtran
                   =e= tariff - netsub + itaxt + ytaxt + ltaxt + ktaxt
                       + gfbor*er ;

   tareq..         tariff =e= sum(inq,pm(inq)*tm(inq)*m(inq)/(1+tm(inq)))
                              + sum(iqf,pm(iqf)*tm(iqf)*m(iqf)/(1+tm(iqf)))
                              + sum(iqd,pm(iqd)*tm(iqd)*m(iqd)/(1+rhod(iqd)));

   subeq..         netsub =e= sum(ie,pe(ie)*te(ie)*e(ie)/(1+te(ie)));

   qrentdeq(i)..   qrentd(i) =e= qd(i)*(er*(rhod(i) - tm(i))*pwm(i)*m(i));

   qrentfeq(i)..   qrentf(i) =e= er*rhof(i)*pwm(i)*m(i);

   ytaxeq..        ytaxt =e= ytax*(y-ltaxt-ktaxt) ;

   itaxeq..        itaxt =e= sum(i,itax(i)*px(i)*x(i) );

   ltaxeq..        ltaxt =e= ltax*ylab;

   ktaxeq..        ktaxt =e= ktax*ycap;

   saveq..         savings =e= mps*(1-ytax)*(y-ltaxt-ktaxt) + govsav
                               + fsav*er;

   walraseq..      walras =e= savings - sum(i,pq(i)*id(i));


* foreign balance

   bop..  fsav + gfbor + remit =e= sum(im,pwm(im)*m(im))
                                   - sum(ie,pwe(ie)*e(ie))
                                   + sum(iqd, thetad(iqd)*qrentd(iqd)/er)
                                   + sum(iqf, thetaf(iqf)*qrentf(iqf)/er);


* foreign prices

   pmdefnq(imnq)..  pm(imnq) =e= (1+tm(imnq))*pwm(imnq)*er;

   pmdefqd(iqd)..   pm(iqd) =e= (1+rhod(iqd))*pwm(iqd)*er;

   pmdefqf(iqf)..   pm(iqf) =e= (1+tm(iqf))*(1+rhof(iqf))*pwm(iqf)*er;

   pedef(ie)..      pe(ie) =e= (1+te(ie))*pwe(ie)*er;


* foreign supply and demand functions

   edemand(ied)..  e(ied) =e= econst(ied)*pwe(ied)**tauf(ied) ;

   msupply(ims)..  m(ims) =e= mconst(ims)*pwm(ims)**sigf(ims) ;


* price index

   pindeq..           pindex =e= gdpva/gdpr ;


* objective function

   obj..             omega =e= ylab + ycap ;


* ## end equations declaration


* ## initialize variable levels ##


 cd.l(i)  = cd0(i);        ds.l(i) = ds0(i);      e.l(i) = e0(i);
 er.l = er0;               fsav.l   = fsav0;      gd.l(i) = gd0(i);
 gdtot.l = gdtot0;         gdpr.l = gdpr0;        gdpva.l = gdpva0;
 govsav.l = govsav0;       gtran.l = gtran0;      id.l(i) = id0(i);
 inter.l(i) = inter0(i);   itaxt.l  = itaxt0;     invest.l = invest0;
 kd.l(i) = kd0(i);         ks.l = ks0;            ktaxt.l = ktaxt0;
 ld.l(i)  = ld0(i);        lesp.l = lesp0;
 ls.l = ls0;               ltaxt.l = ltaxt0;      m.l(i) = m0(i);
 netsub.l = netsub0;       omega.l  = omega0;     px.l(i) = px0(i);
 pd.l(i) = pd0(i);         pe.l(i) = pe0(i);      pindex.l = pindex0;
 pm.l(i) = pm0(i);         pq.l(i)  = pq0(i);     pva.l(i) = pva0(i);
 pva.l(i) = pva0(i);       pwe.l(i) = pwe0(i);    pwe.l(i) = pwe0(i);
 pwm.l(i) = pwm0(i);       q.l(i) = q0(i);        qrentd.l(i) = qrentd0(i);
 qrentf.l(i) = qrentf0(i); ra.l = ra0;            savings.l = savings0;
 tariff.l = tariff0;       wa.l = wa0;            walras.l = 0;
 x.l(i) = x0(i);           y.l = y0;              ycap.l = ycap0;
 ytaxt.l = ytaxt0;         ylab.l = ylab0;
  

* ## set lower bounds for variables ##

 cd.lo(i) = 0;             ds.lo(i) = 0;           e.lo(i) = 0;
 er.lo = lb*er0;           fsav.lo = -ub*fsav0;    gd.lo(i) = 0;
 gdpr.lo = lb*gdpr0;       gdpva.lo = lb*gdpva0;   govsav.lo = ub*govsav0;
 gtran.lo = lb*gtran0;
*id.lo(i) = 0;
 inter.lo(i) = 0;          itaxt.lo = 0;           kd.lo(i) = 0;
 ks.lo = lb*ks0;           ktaxt.lo = lb*ktaxt0;   ld.lo(i) = 0;
 lesp.lo = 0;              ls.lo = lb*ls0;         ltaxt.lo = lb*ltaxt0;
 m.lo(i) = lb*m0(i);       netsub.lo = 0;          omega.lo = lb*omega0;
 px.lo(i) = lb*px0(i);     pd.lo(i) = lb*pd0(i);   pe.lo(i) = lb*pe0(i);
 pindex.lo = lb*pindex0;   pm.lo(i) = lb*pm0(i);   pq.lo(i) = lb*pq0(i);
 pva.lo(i) = lb*pva0(i);   pva.lo(i) = lb*pva0(i); pwe.lo(i)=lb*pwe0(i);
 pwm.lo(i) = lb*pwm0(i);   q.lo(i) = lb*q0(i);     qrentd.lo(i) = 0;
 qrentf.lo(i) = 0;         ra.lo = lb*ra0;         savings.lo = lb*savings0;
 tariff.lo = 0;            wa.lo = lb*wa0;         walras.lo = -savings0;
 x.lo(i) = lb*x0(i);       y.lo = lb*y0;           ycap.lo = lb*ycap0;
 ylab.lo = lb*ylab0;       ytaxt.lo = 0;
  
 
* ## set upper bounds for variables ##

 cd.up(i) = ub*cd0(i);         ds.up(i) = ub*ds0(i);
 e.up(i) = ub*e0(i);           er.up = ub*er0;
 fsav.up = ub*fsav0;           gd.up(i) = ub*gd0(i);
 gdpva.up = ub*gdpva0;         govsav.up = -ub*govsav0;
 gtran.up = ub*gtran0;
*id.up(i) = ub*id0(i);
 inter.up(i) = ub*inter0(i);   itaxt.up = ub*itaxt0;
 kd.up(i) = ub*kd0(i);         ks.up = ub*ks0;
 ktaxt.up = ub*ktaxt0;         ld.up(i) = ub*ld0(i);
 lesp.up = ub*lesp0;           ls.up = ub*ls0;
 ltaxt.up = ub*ltaxt0;         m.up(i) = ub*m0(i);
 netsub.up = ub*netsub0;       omega.up = ub*omega0;
 px.up(i) = ub*px0(i);         pd.up(i) = ub*pd0(i);
 pe.up(i) = ub*pe0(i);         pindex.up  = ub*pindex0;
 pm.up(i) = ub*pm0(i);         pq.up(i) = ub*pq0(i);
 qrentd.up(i) = ub*qrentd0(i); qrentf.up(i) = ub*qrentf0(i);
 ra.up = ub*ra0;               pva.up(i) = ub*pva0(i);
 pva.up(i) = ub*pva0(i);       pwe.up(i)=ub*pwe0(i);
 pwm.up(i) = ub*pwm0(i);       q.up(i) = ub*q0(i);
 gdpr.up = ub*gdpr0;           savings.up = ub*savings0;
 tariff.up = ub*tariff0;       wa.up = ub*wa0;
 walras.up = savings0;         x.up(i) = ub*x0(i);
 y.up = ub*y0;                 ycap.up = ub*ycap0;
 ylab.up = ub*ylab0;           ytaxt.up = ub*ytaxt0;


* ## fix certain trade variables ##

 e.fx(ine)    = 0;
 m.fx(inm)    = 0;
 pm.fx(inm)   = pm0(inm);
 pe.fx(ine)   = pe0(ine);
 pwe.fx(ined) = pwe.l(ined);
 pwm.fx(inms) = pwm.l(inms);
 rhod.fx(im)  = rhod0(im);
 rhof.fx(im)  = rhof0(im);


* ## model closure ##

* foreign exchange closure

* er.fx = er.l ;
  fsav.fx = fsav.l ;

* macro closure

  pindex.fx = pindex.l ;
  invest.fx = invest.l ;
* gtran.fx = gtran.l ;

* government budget closure

   gdtot.fx = gdtot0 ;
*  govsav.fx = govsav.l ;

* labor market closure

*  wa.fx = wa.l ;
   ld.fx(i)$( ld.l(i) eq 0 ) = 0 ;
*  ls.fx = ls.l ;

* capital market closure

*  ra.fx = ra.l ;
*  kd.fx(i) = kd.l(i) ;
   ks.fx = ks.l ;



* ## end of model ##



* ## compute the base equilibrium solution

 options iterlim=3000,limrow=40,limcol=0,domlim=0;
 options solprint=off;

 model generic  / all / ;

 generic.optfile = 1

 solve generic maximizing omega using nlp;

 display walras.l ;

$include define.inc

$include base.inc

 display rscal1,rprice1,rquant1;



